'use strict';

function _interopDefault(ex) {
  return ex && typeof ex === 'object' && 'default' in ex ? ex['default'] : ex;
}

var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var child_process = _interopDefault(require('child_process'));
var stream = _interopDefault(require('stream'));
var assert = _interopDefault(require('assert'));
var events = _interopDefault(require('events'));
var util = _interopDefault(require('util'));

var commonjsGlobal =
  typeof window !== 'undefined'
    ? window
    : typeof global !== 'undefined'
      ? global
      : typeof self !== 'undefined'
        ? self
        : {};

function createCommonjsModule(fn, module) {
  return (module = { exports: {} }), fn(module, module.exports), module.exports;
}

/**
 * Tries to execute a function and discards any error that occurs.
 * @param {Function} fn - Function that might or might not throw an error.
 * @returns {?*} Return-value of the function when no error occurred.
 */

var src = function(fn) {
  try {
    return fn();
  } catch (e) {}
};

var windows = isexe;
isexe.sync = sync;

function checkPathExt(path$$1, options) {
  var pathext =
    options.pathExt !== undefined ? options.pathExt : process.env.PATHEXT;

  if (!pathext) {
    return true;
  }

  pathext = pathext.split(';');

  if (pathext.indexOf('') !== -1) {
    return true;
  }

  for (var i = 0; i < pathext.length; i++) {
    var p = pathext[i].toLowerCase();

    if (p && path$$1.substr(-p.length).toLowerCase() === p) {
      return true;
    }
  }

  return false;
}

function checkStat(stat, path$$1, options) {
  if (!stat.isSymbolicLink() && !stat.isFile()) {
    return false;
  }

  return checkPathExt(path$$1, options);
}

function isexe(path$$1, options, cb) {
  fs.stat(path$$1, function(er, stat) {
    cb(er, er ? false : checkStat(stat, path$$1, options));
  });
}

function sync(path$$1, options) {
  return checkStat(fs.statSync(path$$1), path$$1, options);
}

var mode = isexe$1;
isexe$1.sync = sync$1;

function isexe$1(path$$1, options, cb) {
  fs.stat(path$$1, function(er, stat) {
    cb(er, er ? false : checkStat$1(stat, options));
  });
}

function sync$1(path$$1, options) {
  return checkStat$1(fs.statSync(path$$1), options);
}

function checkStat$1(stat, options) {
  return stat.isFile() && checkMode(stat, options);
}

function checkMode(stat, options) {
  var mod = stat.mode;
  var uid = stat.uid;
  var gid = stat.gid;
  var myUid =
    options.uid !== undefined
      ? options.uid
      : process.getuid && process.getuid();
  var myGid =
    options.gid !== undefined
      ? options.gid
      : process.getgid && process.getgid();
  var u = parseInt('100', 8);
  var g = parseInt('010', 8);
  var o = parseInt('001', 8);
  var ug = u | g;
  var ret =
    mod & o ||
    (mod & g && gid === myGid) ||
    (mod & u && uid === myUid) ||
    (mod & ug && myUid === 0);
  return ret;
}

var core;

if (process.platform === 'win32' || commonjsGlobal.TESTING_WINDOWS) {
  core = windows;
} else {
  core = mode;
}

var isexe_1 = isexe$2;
isexe$2.sync = sync$2;

function isexe$2(path$$1, options, cb) {
  if (typeof options === 'function') {
    cb = options;
    options = {};
  }

  if (!cb) {
    if (typeof Promise !== 'function') {
      throw new TypeError('callback not provided');
    }

    return new Promise(function(resolve, reject) {
      isexe$2(path$$1, options || {}, function(er, is) {
        if (er) {
          reject(er);
        } else {
          resolve(is);
        }
      });
    });
  }

  core(path$$1, options || {}, function(er, is) {
    // ignore EACCES because that just means we aren't allowed to run it
    if (er) {
      if (er.code === 'EACCES' || (options && options.ignoreErrors)) {
        er = null;
        is = false;
      }
    }

    cb(er, is);
  });
}

function sync$2(path$$1, options) {
  // my kingdom for a filtered catch
  try {
    return core.sync(path$$1, options || {});
  } catch (er) {
    if ((options && options.ignoreErrors) || er.code === 'EACCES') {
      return false;
    } else {
      throw er;
    }
  }
}

var which_1 = which;
which.sync = whichSync;
var isWindows =
  process.platform === 'win32' ||
  process.env.OSTYPE === 'cygwin' ||
  process.env.OSTYPE === 'msys';
var COLON = isWindows ? ';' : ':';

function getNotFoundError(cmd) {
  var er = new Error('not found: ' + cmd);
  er.code = 'ENOENT';
  return er;
}

function getPathInfo(cmd, opt) {
  var colon = opt.colon || COLON;
  var pathEnv = opt.path || process.env.PATH || '';
  var pathExt = [''];
  pathEnv = pathEnv.split(colon);
  var pathExtExe = '';

  if (isWindows) {
    pathEnv.unshift(process.cwd());
    pathExtExe = opt.pathExt || process.env.PATHEXT || '.EXE;.CMD;.BAT;.COM';
    pathExt = pathExtExe.split(colon); // Always test the cmd itself first.  isexe will check to make sure
    // it's found in the pathExt set.

    if (cmd.indexOf('.') !== -1 && pathExt[0] !== '') pathExt.unshift('');
  } // If it has a slash, then we don't bother searching the pathenv.
  // just check the file itself, and that's it.

  if (cmd.match(/\//) || (isWindows && cmd.match(/\\/))) pathEnv = [''];
  return {
    env: pathEnv,
    ext: pathExt,
    extExe: pathExtExe
  };
}

function which(cmd, opt, cb) {
  if (typeof opt === 'function') {
    cb = opt;
    opt = {};
  }

  var info = getPathInfo(cmd, opt);
  var pathEnv = info.env;
  var pathExt = info.ext;
  var pathExtExe = info.extExe;
  var found = [];

  (function F(i, l) {
    if (i === l) {
      if (opt.all && found.length) return cb(null, found);
      else return cb(getNotFoundError(cmd));
    }

    var pathPart = pathEnv[i];
    if (pathPart.charAt(0) === '"' && pathPart.slice(-1) === '"')
      pathPart = pathPart.slice(1, -1);
    var p = path.join(pathPart, cmd);

    if (!pathPart && /^\.[\\\/]/.test(cmd)) {
      p = cmd.slice(0, 2) + p;
    }

    (function E(ii, ll) {
      if (ii === ll) return F(i + 1, l);
      var ext = pathExt[ii];
      isexe_1(
        p + ext,
        {
          pathExt: pathExtExe
        },
        function(er, is) {
          if (!er && is) {
            if (opt.all) found.push(p + ext);
            else return cb(null, p + ext);
          }

          return E(ii + 1, ll);
        }
      );
    })(0, pathExt.length);
  })(0, pathEnv.length);
}

function whichSync(cmd, opt) {
  opt = opt || {};
  var info = getPathInfo(cmd, opt);
  var pathEnv = info.env;
  var pathExt = info.ext;
  var pathExtExe = info.extExe;
  var found = [];

  for (var i = 0, l = pathEnv.length; i < l; i++) {
    var pathPart = pathEnv[i];
    if (pathPart.charAt(0) === '"' && pathPart.slice(-1) === '"')
      pathPart = pathPart.slice(1, -1);
    var p = path.join(pathPart, cmd);

    if (!pathPart && /^\.[\\\/]/.test(cmd)) {
      p = cmd.slice(0, 2) + p;
    }

    for (var j = 0, ll = pathExt.length; j < ll; j++) {
      var cur = p + pathExt[j];
      var is;

      try {
        is = isexe_1.sync(cur, {
          pathExt: pathExtExe
        });

        if (is) {
          if (opt.all) found.push(cur);
          else return cur;
        }
      } catch (ex) {}
    }
  }

  if (opt.all && found.length) return found;
  if (opt.nothrow) return null;
  throw getNotFoundError(cmd);
}

var pathKey = opts => {
  opts = opts || {};
  const env = opts.env || process.env;
  const platform = opts.platform || process.platform;

  if (platform !== 'win32') {
    return 'PATH';
  }

  return Object.keys(env).find(x => x.toUpperCase() === 'PATH') || 'Path';
};

const pathKey$1 = pathKey();

function resolveCommandAttempt(parsed, withoutPathExt) {
  const cwd = process.cwd();
  const hasCustomCwd = parsed.options.cwd != null; // If a custom `cwd` was specified, we need to change the process cwd
  // because `which` will do stat calls but does not support a custom cwd

  if (hasCustomCwd) {
    try {
      process.chdir(parsed.options.cwd);
    } catch (err) {
      /* Empty */
    }
  }

  let resolved;

  try {
    resolved = which_1.sync(parsed.command, {
      path: (parsed.options.env || process.env)[pathKey$1],
      pathExt: withoutPathExt ? path.delimiter : undefined
    });
  } catch (e) {
    /* Empty */
  } finally {
    process.chdir(cwd);
  } // If we successfully resolved, ensure that an absolute path is returned
  // Note that when a custom `cwd` was used, we need to resolve to an absolute path based on it

  if (resolved) {
    resolved = path.resolve(hasCustomCwd ? parsed.options.cwd : '', resolved);
  }

  return resolved;
}

function resolveCommand(parsed) {
  return resolveCommandAttempt(parsed) || resolveCommandAttempt(parsed, true);
}

var resolveCommand_1 = resolveCommand;

const metaCharsRegExp = /([()\][%!^"`<>&|;, *?])/g;

function escapeCommand(arg) {
  // Escape meta chars
  arg = arg.replace(metaCharsRegExp, '^$1');
  return arg;
}

function escapeArgument(arg, doubleEscapeMetaChars) {
  // Convert to string
  arg = `${arg}`; // Algorithm below is based on https://qntm.org/cmd
  // Sequence of backslashes followed by a double quote:
  // double up all the backslashes and escape the double quote

  arg = arg.replace(/(\\*)"/g, '$1$1\\"'); // Sequence of backslashes followed by the end of the string
  // (which will become a double quote later):
  // double up all the backslashes

  arg = arg.replace(/(\\*)$/, '$1$1'); // All other backslashes occur literally
  // Quote the whole thing:

  arg = `"${arg}"`; // Escape meta chars

  arg = arg.replace(metaCharsRegExp, '^$1'); // Double escape meta chars if necessary

  if (doubleEscapeMetaChars) {
    arg = arg.replace(metaCharsRegExp, '^$1');
  }

  return arg;
}

var command = escapeCommand;
var argument = escapeArgument;
var _escape = {
  command: command,
  argument: argument
};

var shebangRegex = /^#!.*/;

var shebangCommand = function(str) {
  var match = str.match(shebangRegex);

  if (!match) {
    return null;
  }

  var arr = match[0].replace(/#! ?/, '').split(' ');
  var bin = arr[0].split('/').pop();
  var arg = arr[1];
  return bin === 'env' ? arg : bin + (arg ? ' ' + arg : '');
};

function readShebang(command) {
  // Read the first 150 bytes from the file
  const size = 150;
  let buffer;

  if (Buffer.alloc) {
    // Node.js v4.5+ / v5.10+
    buffer = Buffer.alloc(size);
  } else {
    // Old Node.js API
    buffer = new Buffer(size);
    buffer.fill(0); // zero-fill
  }

  let fd;

  try {
    fd = fs.openSync(command, 'r');
    fs.readSync(fd, buffer, 0, size, 0);
    fs.closeSync(fd);
  } catch (e) {}
  /* Empty */
  // Attempt to extract shebang (null is returned if not a shebang)

  return shebangCommand(buffer.toString());
}

var readShebang_1 = readShebang;

var semver = createCommonjsModule(function(module, exports) {
  exports = module.exports = SemVer; // The debug function is excluded entirely from the minified version.

  /* nomin */

  var debug;
  /* nomin */

  if (
    typeof process === 'object' &&
    /* nomin */
    process.env &&
    /* nomin */
    process.env.NODE_DEBUG &&
    /* nomin */
    /\bsemver\b/i.test(process.env.NODE_DEBUG)
  )
    /* nomin */
    debug = function() {
      /* nomin */
      var args = Array.prototype.slice.call(arguments, 0);
      /* nomin */

      args.unshift('SEMVER');
      /* nomin */

      console.log.apply(console, args);
      /* nomin */
    };
  /* nomin */
  /* nomin */ else debug = function() {}; // Note: this is the semver.org version of the spec that it implements
  // Not necessarily the package version of this code.

  exports.SEMVER_SPEC_VERSION = '2.0.0';
  var MAX_LENGTH = 256;
  var MAX_SAFE_INTEGER = Number.MAX_SAFE_INTEGER || 9007199254740991; // Max safe segment length for coercion.

  var MAX_SAFE_COMPONENT_LENGTH = 16; // The actual regexps go on exports.re

  var re = (exports.re = []);
  var src = (exports.src = []);
  var R = 0; // The following Regular Expressions can be used for tokenizing,
  // validating, and parsing SemVer version strings.
  // ## Numeric Identifier
  // A single `0`, or a non-zero digit followed by zero or more digits.

  var NUMERICIDENTIFIER = R++;
  src[NUMERICIDENTIFIER] = '0|[1-9]\\d*';
  var NUMERICIDENTIFIERLOOSE = R++;
  src[NUMERICIDENTIFIERLOOSE] = '[0-9]+'; // ## Non-numeric Identifier
  // Zero or more digits, followed by a letter or hyphen, and then zero or
  // more letters, digits, or hyphens.

  var NONNUMERICIDENTIFIER = R++;
  src[NONNUMERICIDENTIFIER] = '\\d*[a-zA-Z-][a-zA-Z0-9-]*'; // ## Main Version
  // Three dot-separated numeric identifiers.

  var MAINVERSION = R++;
  src[MAINVERSION] =
    '(' +
    src[NUMERICIDENTIFIER] +
    ')\\.' +
    '(' +
    src[NUMERICIDENTIFIER] +
    ')\\.' +
    '(' +
    src[NUMERICIDENTIFIER] +
    ')';
  var MAINVERSIONLOOSE = R++;
  src[MAINVERSIONLOOSE] =
    '(' +
    src[NUMERICIDENTIFIERLOOSE] +
    ')\\.' +
    '(' +
    src[NUMERICIDENTIFIERLOOSE] +
    ')\\.' +
    '(' +
    src[NUMERICIDENTIFIERLOOSE] +
    ')'; // ## Pre-release Version Identifier
  // A numeric identifier, or a non-numeric identifier.

  var PRERELEASEIDENTIFIER = R++;
  src[PRERELEASEIDENTIFIER] =
    '(?:' + src[NUMERICIDENTIFIER] + '|' + src[NONNUMERICIDENTIFIER] + ')';
  var PRERELEASEIDENTIFIERLOOSE = R++;
  src[PRERELEASEIDENTIFIERLOOSE] =
    '(?:' + src[NUMERICIDENTIFIERLOOSE] + '|' + src[NONNUMERICIDENTIFIER] + ')'; // ## Pre-release Version
  // Hyphen, followed by one or more dot-separated pre-release version
  // identifiers.

  var PRERELEASE = R++;
  src[PRERELEASE] =
    '(?:-(' +
    src[PRERELEASEIDENTIFIER] +
    '(?:\\.' +
    src[PRERELEASEIDENTIFIER] +
    ')*))';
  var PRERELEASELOOSE = R++;
  src[PRERELEASELOOSE] =
    '(?:-?(' +
    src[PRERELEASEIDENTIFIERLOOSE] +
    '(?:\\.' +
    src[PRERELEASEIDENTIFIERLOOSE] +
    ')*))'; // ## Build Metadata Identifier
  // Any combination of digits, letters, or hyphens.

  var BUILDIDENTIFIER = R++;
  src[BUILDIDENTIFIER] = '[0-9A-Za-z-]+'; // ## Build Metadata
  // Plus sign, followed by one or more period-separated build metadata
  // identifiers.

  var BUILD = R++;
  src[BUILD] =
    '(?:\\+(' + src[BUILDIDENTIFIER] + '(?:\\.' + src[BUILDIDENTIFIER] + ')*))'; // ## Full Version String
  // A main version, followed optionally by a pre-release version and
  // build metadata.
  // Note that the only major, minor, patch, and pre-release sections of
  // the version string are capturing groups.  The build metadata is not a
  // capturing group, because it should not ever be used in version
  // comparison.

  var FULL = R++;
  var FULLPLAIN =
    'v?' + src[MAINVERSION] + src[PRERELEASE] + '?' + src[BUILD] + '?';
  src[FULL] = '^' + FULLPLAIN + '$'; // like full, but allows v1.2.3 and =1.2.3, which people do sometimes.
  // also, 1.0.0alpha1 (prerelease without the hyphen) which is pretty
  // common in the npm registry.

  var LOOSEPLAIN =
    '[v=\\s]*' +
    src[MAINVERSIONLOOSE] +
    src[PRERELEASELOOSE] +
    '?' +
    src[BUILD] +
    '?';
  var LOOSE = R++;
  src[LOOSE] = '^' + LOOSEPLAIN + '$';
  var GTLT = R++;
  src[GTLT] = '((?:<|>)?=?)'; // Something like "2.*" or "1.2.x".
  // Note that "x.x" is a valid xRange identifer, meaning "any version"
  // Only the first item is strictly required.

  var XRANGEIDENTIFIERLOOSE = R++;
  src[XRANGEIDENTIFIERLOOSE] = src[NUMERICIDENTIFIERLOOSE] + '|x|X|\\*';
  var XRANGEIDENTIFIER = R++;
  src[XRANGEIDENTIFIER] = src[NUMERICIDENTIFIER] + '|x|X|\\*';
  var XRANGEPLAIN = R++;
  src[XRANGEPLAIN] =
    '[v=\\s]*(' +
    src[XRANGEIDENTIFIER] +
    ')' +
    '(?:\\.(' +
    src[XRANGEIDENTIFIER] +
    ')' +
    '(?:\\.(' +
    src[XRANGEIDENTIFIER] +
    ')' +
    '(?:' +
    src[PRERELEASE] +
    ')?' +
    src[BUILD] +
    '?' +
    ')?)?';
  var XRANGEPLAINLOOSE = R++;
  src[XRANGEPLAINLOOSE] =
    '[v=\\s]*(' +
    src[XRANGEIDENTIFIERLOOSE] +
    ')' +
    '(?:\\.(' +
    src[XRANGEIDENTIFIERLOOSE] +
    ')' +
    '(?:\\.(' +
    src[XRANGEIDENTIFIERLOOSE] +
    ')' +
    '(?:' +
    src[PRERELEASELOOSE] +
    ')?' +
    src[BUILD] +
    '?' +
    ')?)?';
  var XRANGE = R++;
  src[XRANGE] = '^' + src[GTLT] + '\\s*' + src[XRANGEPLAIN] + '$';
  var XRANGELOOSE = R++;
  src[XRANGELOOSE] = '^' + src[GTLT] + '\\s*' + src[XRANGEPLAINLOOSE] + '$'; // Coercion.
  // Extract anything that could conceivably be a part of a valid semver

  var COERCE = R++;
  src[COERCE] =
    '(?:^|[^\\d])' +
    '(\\d{1,' +
    MAX_SAFE_COMPONENT_LENGTH +
    '})' +
    '(?:\\.(\\d{1,' +
    MAX_SAFE_COMPONENT_LENGTH +
    '}))?' +
    '(?:\\.(\\d{1,' +
    MAX_SAFE_COMPONENT_LENGTH +
    '}))?' +
    '(?:$|[^\\d])'; // Tilde ranges.
  // Meaning is "reasonably at or greater than"

  var LONETILDE = R++;
  src[LONETILDE] = '(?:~>?)';
  var TILDETRIM = R++;
  src[TILDETRIM] = '(\\s*)' + src[LONETILDE] + '\\s+';
  re[TILDETRIM] = new RegExp(src[TILDETRIM], 'g');
  var tildeTrimReplace = '$1~';
  var TILDE = R++;
  src[TILDE] = '^' + src[LONETILDE] + src[XRANGEPLAIN] + '$';
  var TILDELOOSE = R++;
  src[TILDELOOSE] = '^' + src[LONETILDE] + src[XRANGEPLAINLOOSE] + '$'; // Caret ranges.
  // Meaning is "at least and backwards compatible with"

  var LONECARET = R++;
  src[LONECARET] = '(?:\\^)';
  var CARETTRIM = R++;
  src[CARETTRIM] = '(\\s*)' + src[LONECARET] + '\\s+';
  re[CARETTRIM] = new RegExp(src[CARETTRIM], 'g');
  var caretTrimReplace = '$1^';
  var CARET = R++;
  src[CARET] = '^' + src[LONECARET] + src[XRANGEPLAIN] + '$';
  var CARETLOOSE = R++;
  src[CARETLOOSE] = '^' + src[LONECARET] + src[XRANGEPLAINLOOSE] + '$'; // A simple gt/lt/eq thing, or just "" to indicate "any version"

  var COMPARATORLOOSE = R++;
  src[COMPARATORLOOSE] = '^' + src[GTLT] + '\\s*(' + LOOSEPLAIN + ')$|^$';
  var COMPARATOR = R++;
  src[COMPARATOR] = '^' + src[GTLT] + '\\s*(' + FULLPLAIN + ')$|^$'; // An expression to strip any whitespace between the gtlt and the thing
  // it modifies, so that `> 1.2.3` ==> `>1.2.3`

  var COMPARATORTRIM = R++;
  src[COMPARATORTRIM] =
    '(\\s*)' + src[GTLT] + '\\s*(' + LOOSEPLAIN + '|' + src[XRANGEPLAIN] + ')'; // this one has to use the /g flag

  re[COMPARATORTRIM] = new RegExp(src[COMPARATORTRIM], 'g');
  var comparatorTrimReplace = '$1$2$3'; // Something like `1.2.3 - 1.2.4`
  // Note that these all use the loose form, because they'll be
  // checked against either the strict or loose comparator form
  // later.

  var HYPHENRANGE = R++;
  src[HYPHENRANGE] =
    '^\\s*(' +
    src[XRANGEPLAIN] +
    ')' +
    '\\s+-\\s+' +
    '(' +
    src[XRANGEPLAIN] +
    ')' +
    '\\s*$';
  var HYPHENRANGELOOSE = R++;
  src[HYPHENRANGELOOSE] =
    '^\\s*(' +
    src[XRANGEPLAINLOOSE] +
    ')' +
    '\\s+-\\s+' +
    '(' +
    src[XRANGEPLAINLOOSE] +
    ')' +
    '\\s*$'; // Star ranges basically just allow anything at all.

  var STAR = R++;
  src[STAR] = '(<|>)?=?\\s*\\*'; // Compile to actual regexp objects.
  // All are flag-free, unless they were created above with a flag.

  for (var i = 0; i < R; i++) {
    debug(i, src[i]);
    if (!re[i]) re[i] = new RegExp(src[i]);
  }

  exports.parse = parse;

  function parse(version, loose) {
    if (version instanceof SemVer) return version;
    if (typeof version !== 'string') return null;
    if (version.length > MAX_LENGTH) return null;
    var r = loose ? re[LOOSE] : re[FULL];
    if (!r.test(version)) return null;

    try {
      return new SemVer(version, loose);
    } catch (er) {
      return null;
    }
  }

  exports.valid = valid;

  function valid(version, loose) {
    var v = parse(version, loose);
    return v ? v.version : null;
  }

  exports.clean = clean;

  function clean(version, loose) {
    var s = parse(version.trim().replace(/^[=v]+/, ''), loose);
    return s ? s.version : null;
  }

  exports.SemVer = SemVer;

  function SemVer(version, loose) {
    if (version instanceof SemVer) {
      if (version.loose === loose) return version;
      else version = version.version;
    } else if (typeof version !== 'string') {
      throw new TypeError('Invalid Version: ' + version);
    }

    if (version.length > MAX_LENGTH)
      throw new TypeError(
        'version is longer than ' + MAX_LENGTH + ' characters'
      );
    if (!(this instanceof SemVer)) return new SemVer(version, loose);
    debug('SemVer', version, loose);
    this.loose = loose;
    var m = version.trim().match(loose ? re[LOOSE] : re[FULL]);
    if (!m) throw new TypeError('Invalid Version: ' + version);
    this.raw = version; // these are actually numbers

    this.major = +m[1];
    this.minor = +m[2];
    this.patch = +m[3];
    if (this.major > MAX_SAFE_INTEGER || this.major < 0)
      throw new TypeError('Invalid major version');
    if (this.minor > MAX_SAFE_INTEGER || this.minor < 0)
      throw new TypeError('Invalid minor version');
    if (this.patch > MAX_SAFE_INTEGER || this.patch < 0)
      throw new TypeError('Invalid patch version'); // numberify any prerelease numeric ids

    if (!m[4]) this.prerelease = [];
    else
      this.prerelease = m[4].split('.').map(function(id) {
        if (/^[0-9]+$/.test(id)) {
          var num = +id;
          if (num >= 0 && num < MAX_SAFE_INTEGER) return num;
        }

        return id;
      });
    this.build = m[5] ? m[5].split('.') : [];
    this.format();
  }

  SemVer.prototype.format = function() {
    this.version = this.major + '.' + this.minor + '.' + this.patch;
    if (this.prerelease.length) this.version += '-' + this.prerelease.join('.');
    return this.version;
  };

  SemVer.prototype.toString = function() {
    return this.version;
  };

  SemVer.prototype.compare = function(other) {
    debug('SemVer.compare', this.version, this.loose, other);
    if (!(other instanceof SemVer)) other = new SemVer(other, this.loose);
    return this.compareMain(other) || this.comparePre(other);
  };

  SemVer.prototype.compareMain = function(other) {
    if (!(other instanceof SemVer)) other = new SemVer(other, this.loose);
    return (
      compareIdentifiers(this.major, other.major) ||
      compareIdentifiers(this.minor, other.minor) ||
      compareIdentifiers(this.patch, other.patch)
    );
  };

  SemVer.prototype.comparePre = function(other) {
    if (!(other instanceof SemVer)) other = new SemVer(other, this.loose); // NOT having a prerelease is > having one

    if (this.prerelease.length && !other.prerelease.length) return -1;
    else if (!this.prerelease.length && other.prerelease.length) return 1;
    else if (!this.prerelease.length && !other.prerelease.length) return 0;
    var i = 0;

    do {
      var a = this.prerelease[i];
      var b = other.prerelease[i];
      debug('prerelease compare', i, a, b);
      if (a === undefined && b === undefined) return 0;
      else if (b === undefined) return 1;
      else if (a === undefined) return -1;
      else if (a === b) continue;
      else return compareIdentifiers(a, b);
    } while (++i);
  }; // preminor will bump the version up to the next minor release, and immediately
  // down to pre-release. premajor and prepatch work the same way.

  SemVer.prototype.inc = function(release, identifier) {
    switch (release) {
      case 'premajor':
        this.prerelease.length = 0;
        this.patch = 0;
        this.minor = 0;
        this.major++;
        this.inc('pre', identifier);
        break;

      case 'preminor':
        this.prerelease.length = 0;
        this.patch = 0;
        this.minor++;
        this.inc('pre', identifier);
        break;

      case 'prepatch':
        // If this is already a prerelease, it will bump to the next version
        // drop any prereleases that might already exist, since they are not
        // relevant at this point.
        this.prerelease.length = 0;
        this.inc('patch', identifier);
        this.inc('pre', identifier);
        break;
      // If the input is a non-prerelease version, this acts the same as
      // prepatch.

      case 'prerelease':
        if (this.prerelease.length === 0) this.inc('patch', identifier);
        this.inc('pre', identifier);
        break;

      case 'major':
        // If this is a pre-major version, bump up to the same major version.
        // Otherwise increment major.
        // 1.0.0-5 bumps to 1.0.0
        // 1.1.0 bumps to 2.0.0
        if (
          this.minor !== 0 ||
          this.patch !== 0 ||
          this.prerelease.length === 0
        )
          this.major++;
        this.minor = 0;
        this.patch = 0;
        this.prerelease = [];
        break;

      case 'minor':
        // If this is a pre-minor version, bump up to the same minor version.
        // Otherwise increment minor.
        // 1.2.0-5 bumps to 1.2.0
        // 1.2.1 bumps to 1.3.0
        if (this.patch !== 0 || this.prerelease.length === 0) this.minor++;
        this.patch = 0;
        this.prerelease = [];
        break;

      case 'patch':
        // If this is not a pre-release version, it will increment the patch.
        // If it is a pre-release it will bump up to the same patch version.
        // 1.2.0-5 patches to 1.2.0
        // 1.2.0 patches to 1.2.1
        if (this.prerelease.length === 0) this.patch++;
        this.prerelease = [];
        break;
      // This probably shouldn't be used publicly.
      // 1.0.0 "pre" would become 1.0.0-0 which is the wrong direction.

      case 'pre':
        if (this.prerelease.length === 0) this.prerelease = [0];
        else {
          var i = this.prerelease.length;

          while (--i >= 0) {
            if (typeof this.prerelease[i] === 'number') {
              this.prerelease[i]++;
              i = -2;
            }
          }

          if (i === -1)
            // didn't increment anything
            this.prerelease.push(0);
        }

        if (identifier) {
          // 1.2.0-beta.1 bumps to 1.2.0-beta.2,
          // 1.2.0-beta.fooblz or 1.2.0-beta bumps to 1.2.0-beta.0
          if (this.prerelease[0] === identifier) {
            if (isNaN(this.prerelease[1])) this.prerelease = [identifier, 0];
          } else this.prerelease = [identifier, 0];
        }

        break;

      default:
        throw new Error('invalid increment argument: ' + release);
    }

    this.format();
    this.raw = this.version;
    return this;
  };

  exports.inc = inc;

  function inc(version, release, loose, identifier) {
    if (typeof loose === 'string') {
      identifier = loose;
      loose = undefined;
    }

    try {
      return new SemVer(version, loose).inc(release, identifier).version;
    } catch (er) {
      return null;
    }
  }

  exports.diff = diff;

  function diff(version1, version2) {
    if (eq(version1, version2)) {
      return null;
    } else {
      var v1 = parse(version1);
      var v2 = parse(version2);

      if (v1.prerelease.length || v2.prerelease.length) {
        for (var key in v1) {
          if (key === 'major' || key === 'minor' || key === 'patch') {
            if (v1[key] !== v2[key]) {
              return 'pre' + key;
            }
          }
        }

        return 'prerelease';
      }

      for (var key in v1) {
        if (key === 'major' || key === 'minor' || key === 'patch') {
          if (v1[key] !== v2[key]) {
            return key;
          }
        }
      }
    }
  }

  exports.compareIdentifiers = compareIdentifiers;
  var numeric = /^[0-9]+$/;

  function compareIdentifiers(a, b) {
    var anum = numeric.test(a);
    var bnum = numeric.test(b);

    if (anum && bnum) {
      a = +a;
      b = +b;
    }

    return anum && !bnum ? -1 : bnum && !anum ? 1 : a < b ? -1 : a > b ? 1 : 0;
  }

  exports.rcompareIdentifiers = rcompareIdentifiers;

  function rcompareIdentifiers(a, b) {
    return compareIdentifiers(b, a);
  }

  exports.major = major;

  function major(a, loose) {
    return new SemVer(a, loose).major;
  }

  exports.minor = minor;

  function minor(a, loose) {
    return new SemVer(a, loose).minor;
  }

  exports.patch = patch;

  function patch(a, loose) {
    return new SemVer(a, loose).patch;
  }

  exports.compare = compare;

  function compare(a, b, loose) {
    return new SemVer(a, loose).compare(new SemVer(b, loose));
  }

  exports.compareLoose = compareLoose;

  function compareLoose(a, b) {
    return compare(a, b, true);
  }

  exports.rcompare = rcompare;

  function rcompare(a, b, loose) {
    return compare(b, a, loose);
  }

  exports.sort = sort;

  function sort(list, loose) {
    return list.sort(function(a, b) {
      return exports.compare(a, b, loose);
    });
  }

  exports.rsort = rsort;

  function rsort(list, loose) {
    return list.sort(function(a, b) {
      return exports.rcompare(a, b, loose);
    });
  }

  exports.gt = gt;

  function gt(a, b, loose) {
    return compare(a, b, loose) > 0;
  }

  exports.lt = lt;

  function lt(a, b, loose) {
    return compare(a, b, loose) < 0;
  }

  exports.eq = eq;

  function eq(a, b, loose) {
    return compare(a, b, loose) === 0;
  }

  exports.neq = neq;

  function neq(a, b, loose) {
    return compare(a, b, loose) !== 0;
  }

  exports.gte = gte;

  function gte(a, b, loose) {
    return compare(a, b, loose) >= 0;
  }

  exports.lte = lte;

  function lte(a, b, loose) {
    return compare(a, b, loose) <= 0;
  }

  exports.cmp = cmp;

  function cmp(a, op, b, loose) {
    var ret;

    switch (op) {
      case '===':
        if (typeof a === 'object') a = a.version;
        if (typeof b === 'object') b = b.version;
        ret = a === b;
        break;

      case '!==':
        if (typeof a === 'object') a = a.version;
        if (typeof b === 'object') b = b.version;
        ret = a !== b;
        break;

      case '':
      case '=':
      case '==':
        ret = eq(a, b, loose);
        break;

      case '!=':
        ret = neq(a, b, loose);
        break;

      case '>':
        ret = gt(a, b, loose);
        break;

      case '>=':
        ret = gte(a, b, loose);
        break;

      case '<':
        ret = lt(a, b, loose);
        break;

      case '<=':
        ret = lte(a, b, loose);
        break;

      default:
        throw new TypeError('Invalid operator: ' + op);
    }

    return ret;
  }

  exports.Comparator = Comparator;

  function Comparator(comp, loose) {
    if (comp instanceof Comparator) {
      if (comp.loose === loose) return comp;
      else comp = comp.value;
    }

    if (!(this instanceof Comparator)) return new Comparator(comp, loose);
    debug('comparator', comp, loose);
    this.loose = loose;
    this.parse(comp);
    if (this.semver === ANY) this.value = '';
    else this.value = this.operator + this.semver.version;
    debug('comp', this);
  }

  var ANY = {};

  Comparator.prototype.parse = function(comp) {
    var r = this.loose ? re[COMPARATORLOOSE] : re[COMPARATOR];
    var m = comp.match(r);
    if (!m) throw new TypeError('Invalid comparator: ' + comp);
    this.operator = m[1];
    if (this.operator === '=') this.operator = ''; // if it literally is just '>' or '' then allow anything.

    if (!m[2]) this.semver = ANY;
    else this.semver = new SemVer(m[2], this.loose);
  };

  Comparator.prototype.toString = function() {
    return this.value;
  };

  Comparator.prototype.test = function(version) {
    debug('Comparator.test', version, this.loose);
    if (this.semver === ANY) return true;
    if (typeof version === 'string') version = new SemVer(version, this.loose);
    return cmp(version, this.operator, this.semver, this.loose);
  };

  Comparator.prototype.intersects = function(comp, loose) {
    if (!(comp instanceof Comparator)) {
      throw new TypeError('a Comparator is required');
    }

    var rangeTmp;

    if (this.operator === '') {
      rangeTmp = new Range(comp.value, loose);
      return satisfies(this.value, rangeTmp, loose);
    } else if (comp.operator === '') {
      rangeTmp = new Range(this.value, loose);
      return satisfies(comp.semver, rangeTmp, loose);
    }

    var sameDirectionIncreasing =
      (this.operator === '>=' || this.operator === '>') &&
      (comp.operator === '>=' || comp.operator === '>');
    var sameDirectionDecreasing =
      (this.operator === '<=' || this.operator === '<') &&
      (comp.operator === '<=' || comp.operator === '<');
    var sameSemVer = this.semver.version === comp.semver.version;
    var differentDirectionsInclusive =
      (this.operator === '>=' || this.operator === '<=') &&
      (comp.operator === '>=' || comp.operator === '<=');
    var oppositeDirectionsLessThan =
      cmp(this.semver, '<', comp.semver, loose) &&
      (this.operator === '>=' || this.operator === '>') &&
      (comp.operator === '<=' || comp.operator === '<');
    var oppositeDirectionsGreaterThan =
      cmp(this.semver, '>', comp.semver, loose) &&
      (this.operator === '<=' || this.operator === '<') &&
      (comp.operator === '>=' || comp.operator === '>');
    return (
      sameDirectionIncreasing ||
      sameDirectionDecreasing ||
      (sameSemVer && differentDirectionsInclusive) ||
      oppositeDirectionsLessThan ||
      oppositeDirectionsGreaterThan
    );
  };

  exports.Range = Range;

  function Range(range, loose) {
    if (range instanceof Range) {
      if (range.loose === loose) {
        return range;
      } else {
        return new Range(range.raw, loose);
      }
    }

    if (range instanceof Comparator) {
      return new Range(range.value, loose);
    }

    if (!(this instanceof Range)) return new Range(range, loose);
    this.loose = loose; // First, split based on boolean or ||

    this.raw = range;
    this.set = range
      .split(/\s*\|\|\s*/)
      .map(function(range) {
        return this.parseRange(range.trim());
      }, this)
      .filter(function(c) {
        // throw out any that are not relevant for whatever reason
        return c.length;
      });

    if (!this.set.length) {
      throw new TypeError('Invalid SemVer Range: ' + range);
    }

    this.format();
  }

  Range.prototype.format = function() {
    this.range = this.set
      .map(function(comps) {
        return comps.join(' ').trim();
      })
      .join('||')
      .trim();
    return this.range;
  };

  Range.prototype.toString = function() {
    return this.range;
  };

  Range.prototype.parseRange = function(range) {
    var loose = this.loose;
    range = range.trim();
    debug('range', range, loose); // `1.2.3 - 1.2.4` => `>=1.2.3 <=1.2.4`

    var hr = loose ? re[HYPHENRANGELOOSE] : re[HYPHENRANGE];
    range = range.replace(hr, hyphenReplace);
    debug('hyphen replace', range); // `> 1.2.3 < 1.2.5` => `>1.2.3 <1.2.5`

    range = range.replace(re[COMPARATORTRIM], comparatorTrimReplace);
    debug('comparator trim', range, re[COMPARATORTRIM]); // `~ 1.2.3` => `~1.2.3`

    range = range.replace(re[TILDETRIM], tildeTrimReplace); // `^ 1.2.3` => `^1.2.3`

    range = range.replace(re[CARETTRIM], caretTrimReplace); // normalize spaces

    range = range.split(/\s+/).join(' '); // At this point, the range is completely trimmed and
    // ready to be split into comparators.

    var compRe = loose ? re[COMPARATORLOOSE] : re[COMPARATOR];
    var set = range
      .split(' ')
      .map(function(comp) {
        return parseComparator(comp, loose);
      })
      .join(' ')
      .split(/\s+/);

    if (this.loose) {
      // in loose mode, throw out any that are not valid comparators
      set = set.filter(function(comp) {
        return !!comp.match(compRe);
      });
    }

    set = set.map(function(comp) {
      return new Comparator(comp, loose);
    });
    return set;
  };

  Range.prototype.intersects = function(range, loose) {
    if (!(range instanceof Range)) {
      throw new TypeError('a Range is required');
    }

    return this.set.some(function(thisComparators) {
      return thisComparators.every(function(thisComparator) {
        return range.set.some(function(rangeComparators) {
          return rangeComparators.every(function(rangeComparator) {
            return thisComparator.intersects(rangeComparator, loose);
          });
        });
      });
    });
  }; // Mostly just for testing and legacy API reasons

  exports.toComparators = toComparators;

  function toComparators(range, loose) {
    return new Range(range, loose).set.map(function(comp) {
      return comp
        .map(function(c) {
          return c.value;
        })
        .join(' ')
        .trim()
        .split(' ');
    });
  } // comprised of xranges, tildes, stars, and gtlt's at this point.
  // already replaced the hyphen ranges
  // turn into a set of JUST comparators.

  function parseComparator(comp, loose) {
    debug('comp', comp);
    comp = replaceCarets(comp, loose);
    debug('caret', comp);
    comp = replaceTildes(comp, loose);
    debug('tildes', comp);
    comp = replaceXRanges(comp, loose);
    debug('xrange', comp);
    comp = replaceStars(comp, loose);
    debug('stars', comp);
    return comp;
  }

  function isX(id) {
    return !id || id.toLowerCase() === 'x' || id === '*';
  } // ~, ~> --> * (any, kinda silly)
  // ~2, ~2.x, ~2.x.x, ~>2, ~>2.x ~>2.x.x --> >=2.0.0 <3.0.0
  // ~2.0, ~2.0.x, ~>2.0, ~>2.0.x --> >=2.0.0 <2.1.0
  // ~1.2, ~1.2.x, ~>1.2, ~>1.2.x --> >=1.2.0 <1.3.0
  // ~1.2.3, ~>1.2.3 --> >=1.2.3 <1.3.0
  // ~1.2.0, ~>1.2.0 --> >=1.2.0 <1.3.0

  function replaceTildes(comp, loose) {
    return comp
      .trim()
      .split(/\s+/)
      .map(function(comp) {
        return replaceTilde(comp, loose);
      })
      .join(' ');
  }

  function replaceTilde(comp, loose) {
    var r = loose ? re[TILDELOOSE] : re[TILDE];
    return comp.replace(r, function(_, M, m, p, pr) {
      debug('tilde', comp, _, M, m, p, pr);
      var ret;
      if (isX(M)) ret = '';
      else if (isX(m)) ret = '>=' + M + '.0.0 <' + (+M + 1) + '.0.0';
      else if (isX(p))
        // ~1.2 == >=1.2.0 <1.3.0
        ret = '>=' + M + '.' + m + '.0 <' + M + '.' + (+m + 1) + '.0';
      else if (pr) {
        debug('replaceTilde pr', pr);
        if (pr.charAt(0) !== '-') pr = '-' + pr;
        ret =
          '>=' + M + '.' + m + '.' + p + pr + ' <' + M + '.' + (+m + 1) + '.0';
      } // ~1.2.3 == >=1.2.3 <1.3.0
      else
        ret = '>=' + M + '.' + m + '.' + p + ' <' + M + '.' + (+m + 1) + '.0';
      debug('tilde return', ret);
      return ret;
    });
  } // ^ --> * (any, kinda silly)
  // ^2, ^2.x, ^2.x.x --> >=2.0.0 <3.0.0
  // ^2.0, ^2.0.x --> >=2.0.0 <3.0.0
  // ^1.2, ^1.2.x --> >=1.2.0 <2.0.0
  // ^1.2.3 --> >=1.2.3 <2.0.0
  // ^1.2.0 --> >=1.2.0 <2.0.0

  function replaceCarets(comp, loose) {
    return comp
      .trim()
      .split(/\s+/)
      .map(function(comp) {
        return replaceCaret(comp, loose);
      })
      .join(' ');
  }

  function replaceCaret(comp, loose) {
    debug('caret', comp, loose);
    var r = loose ? re[CARETLOOSE] : re[CARET];
    return comp.replace(r, function(_, M, m, p, pr) {
      debug('caret', comp, _, M, m, p, pr);
      var ret;
      if (isX(M)) ret = '';
      else if (isX(m)) ret = '>=' + M + '.0.0 <' + (+M + 1) + '.0.0';
      else if (isX(p)) {
        if (M === '0')
          ret = '>=' + M + '.' + m + '.0 <' + M + '.' + (+m + 1) + '.0';
        else ret = '>=' + M + '.' + m + '.0 <' + (+M + 1) + '.0.0';
      } else if (pr) {
        debug('replaceCaret pr', pr);
        if (pr.charAt(0) !== '-') pr = '-' + pr;

        if (M === '0') {
          if (m === '0')
            ret =
              '>=' +
              M +
              '.' +
              m +
              '.' +
              p +
              pr +
              ' <' +
              M +
              '.' +
              m +
              '.' +
              (+p + 1);
          else
            ret =
              '>=' +
              M +
              '.' +
              m +
              '.' +
              p +
              pr +
              ' <' +
              M +
              '.' +
              (+m + 1) +
              '.0';
        } else
          ret = '>=' + M + '.' + m + '.' + p + pr + ' <' + (+M + 1) + '.0.0';
      } else {
        debug('no pr');

        if (M === '0') {
          if (m === '0')
            ret =
              '>=' +
              M +
              '.' +
              m +
              '.' +
              p +
              ' <' +
              M +
              '.' +
              m +
              '.' +
              (+p + 1);
          else
            ret =
              '>=' + M + '.' + m + '.' + p + ' <' + M + '.' + (+m + 1) + '.0';
        } else ret = '>=' + M + '.' + m + '.' + p + ' <' + (+M + 1) + '.0.0';
      }
      debug('caret return', ret);
      return ret;
    });
  }

  function replaceXRanges(comp, loose) {
    debug('replaceXRanges', comp, loose);
    return comp
      .split(/\s+/)
      .map(function(comp) {
        return replaceXRange(comp, loose);
      })
      .join(' ');
  }

  function replaceXRange(comp, loose) {
    comp = comp.trim();
    var r = loose ? re[XRANGELOOSE] : re[XRANGE];
    return comp.replace(r, function(ret, gtlt, M, m, p, pr) {
      debug('xRange', comp, ret, gtlt, M, m, p, pr);
      var xM = isX(M);
      var xm = xM || isX(m);
      var xp = xm || isX(p);
      var anyX = xp;
      if (gtlt === '=' && anyX) gtlt = '';

      if (xM) {
        if (gtlt === '>' || gtlt === '<') {
          // nothing is allowed
          ret = '<0.0.0';
        } else {
          // nothing is forbidden
          ret = '*';
        }
      } else if (gtlt && anyX) {
        // replace X with 0
        if (xm) m = 0;
        if (xp) p = 0;

        if (gtlt === '>') {
          // >1 => >=2.0.0
          // >1.2 => >=1.3.0
          // >1.2.3 => >= 1.2.4
          gtlt = '>=';

          if (xm) {
            M = +M + 1;
            m = 0;
            p = 0;
          } else if (xp) {
            m = +m + 1;
            p = 0;
          }
        } else if (gtlt === '<=') {
          // <=0.7.x is actually <0.8.0, since any 0.7.x should
          // pass.  Similarly, <=7.x is actually <8.0.0, etc.
          gtlt = '<';
          if (xm) M = +M + 1;
          else m = +m + 1;
        }

        ret = gtlt + M + '.' + m + '.' + p;
      } else if (xm) {
        ret = '>=' + M + '.0.0 <' + (+M + 1) + '.0.0';
      } else if (xp) {
        ret = '>=' + M + '.' + m + '.0 <' + M + '.' + (+m + 1) + '.0';
      }

      debug('xRange return', ret);
      return ret;
    });
  } // Because * is AND-ed with everything else in the comparator,
  // and '' means "any version", just remove the *s entirely.

  function replaceStars(comp, loose) {
    debug('replaceStars', comp, loose); // Looseness is ignored here.  star is always as loose as it gets!

    return comp.trim().replace(re[STAR], '');
  } // This function is passed to string.replace(re[HYPHENRANGE])
  // M, m, patch, prerelease, build
  // 1.2 - 3.4.5 => >=1.2.0 <=3.4.5
  // 1.2.3 - 3.4 => >=1.2.0 <3.5.0 Any 3.4.x will do
  // 1.2 - 3.4 => >=1.2.0 <3.5.0

  function hyphenReplace(
    $0,
    from,
    fM,
    fm,
    fp,
    fpr,
    fb,
    to,
    tM,
    tm,
    tp,
    tpr,
    tb
  ) {
    if (isX(fM)) from = '';
    else if (isX(fm)) from = '>=' + fM + '.0.0';
    else if (isX(fp)) from = '>=' + fM + '.' + fm + '.0';
    else from = '>=' + from;
    if (isX(tM)) to = '';
    else if (isX(tm)) to = '<' + (+tM + 1) + '.0.0';
    else if (isX(tp)) to = '<' + tM + '.' + (+tm + 1) + '.0';
    else if (tpr) to = '<=' + tM + '.' + tm + '.' + tp + '-' + tpr;
    else to = '<=' + to;
    return (from + ' ' + to).trim();
  } // if ANY of the sets match ALL of its comparators, then pass

  Range.prototype.test = function(version) {
    if (!version) return false;
    if (typeof version === 'string') version = new SemVer(version, this.loose);

    for (var i = 0; i < this.set.length; i++) {
      if (testSet(this.set[i], version)) return true;
    }

    return false;
  };

  function testSet(set, version) {
    for (var i = 0; i < set.length; i++) {
      if (!set[i].test(version)) return false;
    }

    if (version.prerelease.length) {
      // Find the set of versions that are allowed to have prereleases
      // For example, ^1.2.3-pr.1 desugars to >=1.2.3-pr.1 <2.0.0
      // That should allow `1.2.3-pr.2` to pass.
      // However, `1.2.4-alpha.notready` should NOT be allowed,
      // even though it's within the range set by the comparators.
      for (var i = 0; i < set.length; i++) {
        debug(set[i].semver);
        if (set[i].semver === ANY) continue;

        if (set[i].semver.prerelease.length > 0) {
          var allowed = set[i].semver;
          if (
            allowed.major === version.major &&
            allowed.minor === version.minor &&
            allowed.patch === version.patch
          )
            return true;
        }
      } // Version has a -pre, but it's not one of the ones we like.

      return false;
    }

    return true;
  }

  exports.satisfies = satisfies;

  function satisfies(version, range, loose) {
    try {
      range = new Range(range, loose);
    } catch (er) {
      return false;
    }

    return range.test(version);
  }

  exports.maxSatisfying = maxSatisfying;

  function maxSatisfying(versions, range, loose) {
    var max = null;
    var maxSV = null;

    try {
      var rangeObj = new Range(range, loose);
    } catch (er) {
      return null;
    }

    versions.forEach(function(v) {
      if (rangeObj.test(v)) {
        // satisfies(v, range, loose)
        if (!max || maxSV.compare(v) === -1) {
          // compare(max, v, true)
          max = v;
          maxSV = new SemVer(max, loose);
        }
      }
    });
    return max;
  }

  exports.minSatisfying = minSatisfying;

  function minSatisfying(versions, range, loose) {
    var min = null;
    var minSV = null;

    try {
      var rangeObj = new Range(range, loose);
    } catch (er) {
      return null;
    }

    versions.forEach(function(v) {
      if (rangeObj.test(v)) {
        // satisfies(v, range, loose)
        if (!min || minSV.compare(v) === 1) {
          // compare(min, v, true)
          min = v;
          minSV = new SemVer(min, loose);
        }
      }
    });
    return min;
  }

  exports.validRange = validRange;

  function validRange(range, loose) {
    try {
      // Return '*' instead of '' so that truthiness works.
      // This will throw if it's invalid anyway
      return new Range(range, loose).range || '*';
    } catch (er) {
      return null;
    }
  } // Determine if version is less than all the versions possible in the range

  exports.ltr = ltr;

  function ltr(version, range, loose) {
    return outside(version, range, '<', loose);
  } // Determine if version is greater than all the versions possible in the range.

  exports.gtr = gtr;

  function gtr(version, range, loose) {
    return outside(version, range, '>', loose);
  }

  exports.outside = outside;

  function outside(version, range, hilo, loose) {
    version = new SemVer(version, loose);
    range = new Range(range, loose);
    var gtfn, ltefn, ltfn, comp, ecomp;

    switch (hilo) {
      case '>':
        gtfn = gt;
        ltefn = lte;
        ltfn = lt;
        comp = '>';
        ecomp = '>=';
        break;

      case '<':
        gtfn = lt;
        ltefn = gte;
        ltfn = gt;
        comp = '<';
        ecomp = '<=';
        break;

      default:
        throw new TypeError('Must provide a hilo val of "<" or ">"');
    } // If it satisifes the range it is not outside

    if (satisfies(version, range, loose)) {
      return false;
    } // From now on, variable terms are as if we're in "gtr" mode.
    // but note that everything is flipped for the "ltr" function.

    for (var i = 0; i < range.set.length; ++i) {
      var comparators = range.set[i];
      var high = null;
      var low = null;
      comparators.forEach(function(comparator) {
        if (comparator.semver === ANY) {
          comparator = new Comparator('>=0.0.0');
        }

        high = high || comparator;
        low = low || comparator;

        if (gtfn(comparator.semver, high.semver, loose)) {
          high = comparator;
        } else if (ltfn(comparator.semver, low.semver, loose)) {
          low = comparator;
        }
      }); // If the edge version comparator has a operator then our version
      // isn't outside it

      if (high.operator === comp || high.operator === ecomp) {
        return false;
      } // If the lowest version comparator has an operator and our version
      // is less than it then it isn't higher than the range

      if (
        (!low.operator || low.operator === comp) &&
        ltefn(version, low.semver)
      ) {
        return false;
      } else if (low.operator === ecomp && ltfn(version, low.semver)) {
        return false;
      }
    }

    return true;
  }

  exports.prerelease = prerelease;

  function prerelease(version, loose) {
    var parsed = parse(version, loose);
    return parsed && parsed.prerelease.length ? parsed.prerelease : null;
  }

  exports.intersects = intersects;

  function intersects(r1, r2, loose) {
    r1 = new Range(r1, loose);
    r2 = new Range(r2, loose);
    return r1.intersects(r2);
  }

  exports.coerce = coerce;

  function coerce(version) {
    if (version instanceof SemVer) return version;
    if (typeof version !== 'string') return null;
    var match = version.match(re[COERCE]);
    if (match == null) return null;
    return parse(
      (match[1] || '0') + '.' + (match[2] || '0') + '.' + (match[3] || '0')
    );
  }
});
var semver_1 = semver.SEMVER_SPEC_VERSION;
var semver_2 = semver.re;
var semver_3 = semver.src;
var semver_4 = semver.parse;
var semver_5 = semver.valid;
var semver_6 = semver.clean;
var semver_7 = semver.SemVer;
var semver_8 = semver.inc;
var semver_9 = semver.diff;
var semver_10 = semver.compareIdentifiers;
var semver_11 = semver.rcompareIdentifiers;
var semver_12 = semver.major;
var semver_13 = semver.minor;
var semver_14 = semver.patch;
var semver_15 = semver.compare;
var semver_16 = semver.compareLoose;
var semver_17 = semver.rcompare;
var semver_18 = semver.sort;
var semver_19 = semver.rsort;
var semver_20 = semver.gt;
var semver_21 = semver.lt;
var semver_22 = semver.eq;
var semver_23 = semver.neq;
var semver_24 = semver.gte;
var semver_25 = semver.lte;
var semver_26 = semver.cmp;
var semver_27 = semver.Comparator;
var semver_28 = semver.Range;
var semver_29 = semver.toComparators;
var semver_30 = semver.satisfies;
var semver_31 = semver.maxSatisfying;
var semver_32 = semver.minSatisfying;
var semver_33 = semver.validRange;
var semver_34 = semver.ltr;
var semver_35 = semver.gtr;
var semver_36 = semver.outside;
var semver_37 = semver.prerelease;
var semver_38 = semver.intersects;
var semver_39 = semver.coerce;

const isWin = process.platform === 'win32';
const isExecutableRegExp = /\.(?:com|exe)$/i;
const isCmdShimRegExp = /node_modules[\\/].bin[\\/][^\\/]+\.cmd$/i; // `options.shell` is supported in Node ^4.8.0, ^5.7.0 and >= 6.0.0

const supportsShellOption =
  src(() =>
    semver.satisfies(process.version, '^4.8.0 || ^5.7.0 || >= 6.0.0', true)
  ) || false;

function detectShebang(parsed) {
  parsed.file = resolveCommand_1(parsed);
  const shebang = parsed.file && readShebang_1(parsed.file);

  if (shebang) {
    parsed.args.unshift(parsed.file);
    parsed.command = shebang;
    return resolveCommand_1(parsed);
  }

  return parsed.file;
}

function parseNonShell(parsed) {
  if (!isWin) {
    return parsed;
  } // Detect & add support for shebangs

  const commandFile = detectShebang(parsed); // We don't need a shell if the command filename is an executable

  const needsShell = !isExecutableRegExp.test(commandFile); // If a shell is required, use cmd.exe and take care of escaping everything correctly
  // Note that `forceShell` is an hidden option used only in tests

  if (parsed.options.forceShell || needsShell) {
    // Need to double escape meta chars if the command is a cmd-shim located in `node_modules/.bin/`
    // The cmd-shim simply calls execute the package bin file with NodeJS, proxying any argument
    // Because the escape of metachars with ^ gets interpreted when the cmd.exe is first called,
    // we need to double escape them
    const needsDoubleEscapeMetaChars = isCmdShimRegExp.test(commandFile); // Normalize posix paths into OS compatible paths (e.g.: foo/bar -> foo\bar)
    // This is necessary otherwise it will always fail with ENOENT in those cases

    parsed.command = path.normalize(parsed.command); // Escape command & arguments

    parsed.command = _escape.command(parsed.command);
    parsed.args = parsed.args.map(arg =>
      _escape.argument(arg, needsDoubleEscapeMetaChars)
    );
    const shellCommand = [parsed.command].concat(parsed.args).join(' ');
    parsed.args = ['/d', '/s', '/c', `"${shellCommand}"`];
    parsed.command = process.env.comspec || 'cmd.exe';
    parsed.options.windowsVerbatimArguments = true; // Tell node's spawn that the arguments are already escaped
  }

  return parsed;
}

function parseShell(parsed) {
  // If node supports the shell option, there's no need to mimic its behavior
  if (supportsShellOption) {
    return parsed;
  } // Mimic node shell option
  // See https://github.com/nodejs/node/blob/b9f6a2dc059a1062776133f3d4fd848c4da7d150/lib/child_process.js#L335

  const shellCommand = [parsed.command].concat(parsed.args).join(' ');

  if (isWin) {
    parsed.command =
      typeof parsed.options.shell === 'string'
        ? parsed.options.shell
        : process.env.comspec || 'cmd.exe';
    parsed.args = ['/d', '/s', '/c', `"${shellCommand}"`];
    parsed.options.windowsVerbatimArguments = true; // Tell node's spawn that the arguments are already escaped
  } else {
    if (typeof parsed.options.shell === 'string') {
      parsed.command = parsed.options.shell;
    } else if (process.platform === 'android') {
      parsed.command = '/system/bin/sh';
    } else {
      parsed.command = '/bin/sh';
    }

    parsed.args = ['-c', shellCommand];
  }

  return parsed;
}

function parse(command, args, options) {
  // Normalize arguments, similar to nodejs
  if (args && !Array.isArray(args)) {
    options = args;
    args = null;
  }

  args = args ? args.slice(0) : []; // Clone array to avoid changing the original

  options = Object.assign({}, options); // Clone object to avoid changing the original
  // Build our parsed object

  const parsed = {
    command,
    args,
    options,
    file: undefined,
    original: {
      command,
      args
    }
  }; // Delegate further parsing to shell or non-shell

  return options.shell ? parseShell(parsed) : parseNonShell(parsed);
}

var parse_1 = parse;

const isWin$1 = process.platform === 'win32';

function notFoundError(original, syscall) {
  return Object.assign(new Error(`${syscall} ${original.command} ENOENT`), {
    code: 'ENOENT',
    errno: 'ENOENT',
    syscall: `${syscall} ${original.command}`,
    path: original.command,
    spawnargs: original.args
  });
}

function hookChildProcess(cp, parsed) {
  if (!isWin$1) {
    return;
  }

  const originalEmit = cp.emit;

  cp.emit = function(name, arg1) {
    // If emitting "exit" event and exit code is 1, we need to check if
    // the command exists and emit an "error" instead
    // See https://github.com/IndigoUnited/node-cross-spawn/issues/16
    if (name === 'exit') {
      const err = verifyENOENT(arg1, parsed, 'spawn');

      if (err) {
        return originalEmit.call(cp, 'error', err);
      }
    }

    return originalEmit.apply(cp, arguments); // eslint-disable-line prefer-rest-params
  };
}

function verifyENOENT(status, parsed) {
  if (isWin$1 && status === 1 && !parsed.file) {
    return notFoundError(parsed.original, 'spawn');
  }

  return null;
}

function verifyENOENTSync(status, parsed) {
  if (isWin$1 && status === 1 && !parsed.file) {
    return notFoundError(parsed.original, 'spawnSync');
  }

  return null;
}

var enoent = {
  hookChildProcess,
  verifyENOENT,
  verifyENOENTSync,
  notFoundError
};

function spawn(command, args, options) {
  // Parse the arguments
  const parsed = parse_1(command, args, options); // Spawn the child process

  const spawned = child_process.spawn(
    parsed.command,
    parsed.args,
    parsed.options
  ); // Hook into child process "exit" event to emit an error if the command
  // does not exists, see: https://github.com/IndigoUnited/node-cross-spawn/issues/16

  enoent.hookChildProcess(spawned, parsed);
  return spawned;
}

function spawnSync(command, args, options) {
  // Parse the arguments
  const parsed = parse_1(command, args, options); // Spawn the child process

  const result = child_process.spawnSync(
    parsed.command,
    parsed.args,
    parsed.options
  ); // Analyze if the command does not exist, see: https://github.com/IndigoUnited/node-cross-spawn/issues/16

  result.error = result.error || enoent.verifyENOENTSync(result.status, parsed);
  return result;
}

var crossSpawn = spawn;
var spawn_1 = spawn;
var sync$3 = spawnSync;
var _parse = parse_1;
var _enoent = enoent;
crossSpawn.spawn = spawn_1;
crossSpawn.sync = sync$3;
crossSpawn._parse = _parse;
crossSpawn._enoent = _enoent;

var stripEof = function(x) {
  var lf = typeof x === 'string' ? '\n' : '\n'.charCodeAt();
  var cr = typeof x === 'string' ? '\r' : '\r'.charCodeAt();

  if (x[x.length - 1] === lf) {
    x = x.slice(0, x.length - 1);
  }

  if (x[x.length - 1] === cr) {
    x = x.slice(0, x.length - 1);
  }

  return x;
};

var npmRunPath = createCommonjsModule(function(module) {
  module.exports = opts => {
    opts = Object.assign(
      {
        cwd: process.cwd(),
        path: process.env[pathKey()]
      },
      opts
    );
    let prev;
    let pth = path.resolve(opts.cwd);
    const ret = [];

    while (prev !== pth) {
      ret.push(path.join(pth, 'node_modules/.bin'));
      prev = pth;
      pth = path.resolve(pth, '..');
    } // ensure the running `node` binary is used

    ret.push(path.dirname(process.execPath));
    return ret.concat(opts.path).join(path.delimiter);
  };

  module.exports.env = opts => {
    opts = Object.assign(
      {
        env: process.env
      },
      opts
    );
    const env = Object.assign({}, opts.env);
    const path$$1 = pathKey({
      env
    });
    opts.path = env[path$$1];
    env[path$$1] = module.exports(opts);
    return env;
  };
});
var npmRunPath_1 = npmRunPath.env;

var isStream_1 = createCommonjsModule(function(module) {
  var isStream = (module.exports = function(stream$$1) {
    return (
      stream$$1 !== null &&
      typeof stream$$1 === 'object' &&
      typeof stream$$1.pipe === 'function'
    );
  });

  isStream.writable = function(stream$$1) {
    return (
      isStream(stream$$1) &&
      stream$$1.writable !== false &&
      typeof stream$$1._write === 'function' &&
      typeof stream$$1._writableState === 'object'
    );
  };

  isStream.readable = function(stream$$1) {
    return (
      isStream(stream$$1) &&
      stream$$1.readable !== false &&
      typeof stream$$1._read === 'function' &&
      typeof stream$$1._readableState === 'object'
    );
  };

  isStream.duplex = function(stream$$1) {
    return isStream.writable(stream$$1) && isStream.readable(stream$$1);
  };

  isStream.transform = function(stream$$1) {
    return (
      isStream.duplex(stream$$1) &&
      typeof stream$$1._transform === 'function' &&
      typeof stream$$1._transformState === 'object'
    );
  };
});

// Returns a wrapper function that returns a wrapped callback
// The wrapper function should do some stuff, and return a
// presumably different callback function.
// This makes sure that own properties are retained, so that
// decorations and such are not lost along the way.
var wrappy_1 = wrappy;

function wrappy(fn, cb) {
  if (fn && cb) return wrappy(fn)(cb);
  if (typeof fn !== 'function') throw new TypeError('need wrapper function');
  Object.keys(fn).forEach(function(k) {
    wrapper[k] = fn[k];
  });
  return wrapper;

  function wrapper() {
    var args = new Array(arguments.length);

    for (var i = 0; i < args.length; i++) {
      args[i] = arguments[i];
    }

    var ret = fn.apply(this, args);
    var cb = args[args.length - 1];

    if (typeof ret === 'function' && ret !== cb) {
      Object.keys(cb).forEach(function(k) {
        ret[k] = cb[k];
      });
    }

    return ret;
  }
}

var once_1 = wrappy_1(once);
var strict = wrappy_1(onceStrict);
once.proto = once(function() {
  Object.defineProperty(Function.prototype, 'once', {
    value: function() {
      return once(this);
    },
    configurable: true
  });
  Object.defineProperty(Function.prototype, 'onceStrict', {
    value: function() {
      return onceStrict(this);
    },
    configurable: true
  });
});

function once(fn) {
  var f = function() {
    if (f.called) return f.value;
    f.called = true;
    return (f.value = fn.apply(this, arguments));
  };

  f.called = false;
  return f;
}

function onceStrict(fn) {
  var f = function() {
    if (f.called) throw new Error(f.onceError);
    f.called = true;
    return (f.value = fn.apply(this, arguments));
  };

  var name = fn.name || 'Function wrapped with `once`';
  f.onceError = name + " shouldn't be called more than once";
  f.called = false;
  return f;
}
once_1.strict = strict;

var noop = function() {};

var isRequest = function(stream$$1) {
  return stream$$1.setHeader && typeof stream$$1.abort === 'function';
};

var isChildProcess = function(stream$$1) {
  return (
    stream$$1.stdio &&
    Array.isArray(stream$$1.stdio) &&
    stream$$1.stdio.length === 3
  );
};

var eos = function(stream$$1, opts, callback) {
  if (typeof opts === 'function') return eos(stream$$1, null, opts);
  if (!opts) opts = {};
  callback = once_1(callback || noop);
  var ws = stream$$1._writableState;
  var rs = stream$$1._readableState;
  var readable =
    opts.readable || (opts.readable !== false && stream$$1.readable);
  var writable =
    opts.writable || (opts.writable !== false && stream$$1.writable);

  var onlegacyfinish = function() {
    if (!stream$$1.writable) onfinish();
  };

  var onfinish = function() {
    writable = false;
    if (!readable) callback.call(stream$$1);
  };

  var onend = function() {
    readable = false;
    if (!writable) callback.call(stream$$1);
  };

  var onexit = function(exitCode) {
    callback.call(
      stream$$1,
      exitCode ? new Error('exited with error code: ' + exitCode) : null
    );
  };

  var onerror = function(err) {
    callback.call(stream$$1, err);
  };

  var onclose = function() {
    if (readable && !(rs && rs.ended))
      return callback.call(stream$$1, new Error('premature close'));
    if (writable && !(ws && ws.ended))
      return callback.call(stream$$1, new Error('premature close'));
  };

  var onrequest = function() {
    stream$$1.req.on('finish', onfinish);
  };

  if (isRequest(stream$$1)) {
    stream$$1.on('complete', onfinish);
    stream$$1.on('abort', onclose);
    if (stream$$1.req) onrequest();
    else stream$$1.on('request', onrequest);
  } else if (writable && !ws) {
    // legacy streams
    stream$$1.on('end', onlegacyfinish);
    stream$$1.on('close', onlegacyfinish);
  }

  if (isChildProcess(stream$$1)) stream$$1.on('exit', onexit);
  stream$$1.on('end', onend);
  stream$$1.on('finish', onfinish);
  if (opts.error !== false) stream$$1.on('error', onerror);
  stream$$1.on('close', onclose);
  return function() {
    stream$$1.removeListener('complete', onfinish);
    stream$$1.removeListener('abort', onclose);
    stream$$1.removeListener('request', onrequest);
    if (stream$$1.req) stream$$1.req.removeListener('finish', onfinish);
    stream$$1.removeListener('end', onlegacyfinish);
    stream$$1.removeListener('close', onlegacyfinish);
    stream$$1.removeListener('finish', onfinish);
    stream$$1.removeListener('exit', onexit);
    stream$$1.removeListener('end', onend);
    stream$$1.removeListener('error', onerror);
    stream$$1.removeListener('close', onclose);
  };
};

var endOfStream = eos;

var noop$1 = function() {};

var ancient = /^v?\.0/.test(process.version);

var isFn = function(fn) {
  return typeof fn === 'function';
};

var isFS = function(stream$$1) {
  if (!ancient) return false; // newer node version do not need to care about fs is a special way

  if (!fs) return false; // browser

  return (
    (stream$$1 instanceof (fs.ReadStream || noop$1) ||
      stream$$1 instanceof (fs.WriteStream || noop$1)) &&
    isFn(stream$$1.close)
  );
};

var isRequest$1 = function(stream$$1) {
  return stream$$1.setHeader && isFn(stream$$1.abort);
};

var destroyer = function(stream$$1, reading, writing, callback) {
  callback = once_1(callback);
  var closed = false;
  stream$$1.on('close', function() {
    closed = true;
  });
  endOfStream(
    stream$$1,
    {
      readable: reading,
      writable: writing
    },
    function(err) {
      if (err) return callback(err);
      closed = true;
      callback();
    }
  );
  var destroyed = false;
  return function(err) {
    if (closed) return;
    if (destroyed) return;
    destroyed = true;
    if (isFS(stream$$1)) return stream$$1.close(noop$1); // use close for fs streams to avoid fd leaks

    if (isRequest$1(stream$$1)) return stream$$1.abort(); // request.destroy just do .end - .abort is what we want

    if (isFn(stream$$1.destroy)) return stream$$1.destroy();
    callback(err || new Error('stream was destroyed'));
  };
};

var call = function(fn) {
  fn();
};

var pipe = function(from, to) {
  return from.pipe(to);
};

var pump = function() {
  var streams = Array.prototype.slice.call(arguments);
  var callback =
    (isFn(streams[streams.length - 1] || noop$1) && streams.pop()) || noop$1;
  if (Array.isArray(streams[0])) streams = streams[0];
  if (streams.length < 2)
    throw new Error('pump requires two streams per minimum');
  var error;
  var destroys = streams.map(function(stream$$1, i) {
    var reading = i < streams.length - 1;
    var writing = i > 0;
    return destroyer(stream$$1, reading, writing, function(err) {
      if (!error) error = err;
      if (err) destroys.forEach(call);
      if (reading) return;
      destroys.forEach(call);
      callback(error);
    });
  });
  return streams.reduce(pipe);
};

var pump_1 = pump;

const { PassThrough } = stream;

var bufferStream = options => {
  options = Object.assign({}, options);
  const { array } = options;
  let { encoding } = options;
  const buffer = encoding === 'buffer';
  let objectMode = false;

  if (array) {
    objectMode = !(encoding || buffer);
  } else {
    encoding = encoding || 'utf8';
  }

  if (buffer) {
    encoding = null;
  }

  let len = 0;
  const ret = [];
  const stream$$1 = new PassThrough({
    objectMode
  });

  if (encoding) {
    stream$$1.setEncoding(encoding);
  }

  stream$$1.on('data', chunk => {
    ret.push(chunk);

    if (objectMode) {
      len = ret.length;
    } else {
      len += chunk.length;
    }
  });

  stream$$1.getBufferedValue = () => {
    if (array) {
      return ret;
    }

    return buffer ? Buffer.concat(ret, len) : ret.join('');
  };

  stream$$1.getBufferedLength = () => len;

  return stream$$1;
};

function getStream(inputStream, options) {
  if (!inputStream) {
    return Promise.reject(new Error('Expected a stream'));
  }

  options = Object.assign(
    {
      maxBuffer: Infinity
    },
    options
  );
  const { maxBuffer } = options;
  let stream$$1;
  return new Promise((resolve, reject) => {
    const rejectPromise = error => {
      if (error) {
        // A null check
        error.bufferedData = stream$$1.getBufferedValue();
      }

      reject(error);
    };

    stream$$1 = pump_1(inputStream, bufferStream(options), error => {
      if (error) {
        rejectPromise(error);
        return;
      }

      resolve();
    });
    stream$$1.on('data', () => {
      if (stream$$1.getBufferedLength() > maxBuffer) {
        rejectPromise(new Error('maxBuffer exceeded'));
      }
    });
  }).then(() => stream$$1.getBufferedValue());
}

var getStream_1 = getStream;

var buffer = (stream$$1, options) =>
  getStream(
    stream$$1,
    Object.assign({}, options, {
      encoding: 'buffer'
    })
  );

var array = (stream$$1, options) =>
  getStream(
    stream$$1,
    Object.assign({}, options, {
      array: true
    })
  );
getStream_1.buffer = buffer;
getStream_1.array = array;

var pFinally = (promise, onFinally) => {
  onFinally = onFinally || (() => {});

  return promise.then(
    val =>
      new Promise(resolve => {
        resolve(onFinally());
      }).then(() => val),
    err =>
      new Promise(resolve => {
        resolve(onFinally());
      }).then(() => {
        throw err;
      })
  );
};

var signals = createCommonjsModule(function(module) {
  // This is not the set of all possible signals.
  //
  // It IS, however, the set of all signals that trigger
  // an exit on either Linux or BSD systems.  Linux is a
  // superset of the signal names supported on BSD, and
  // the unknown signals just fail to register, so we can
  // catch that easily enough.
  //
  // Don't bother with SIGKILL.  It's uncatchable, which
  // means that we can't fire any callbacks anyway.
  //
  // If a user does happen to register a handler on a non-
  // fatal signal like SIGWINCH or something, and then
  // exit, it'll end up firing `process.emit('exit')`, so
  // the handler will be fired anyway.
  //
  // SIGBUS, SIGFPE, SIGSEGV and SIGILL, when not raised
  // artificially, inherently leave the process in a
  // state from which it is not safe to try and enter JS
  // listeners.
  module.exports = ['SIGABRT', 'SIGALRM', 'SIGHUP', 'SIGINT', 'SIGTERM'];

  if (process.platform !== 'win32') {
    module.exports.push(
      'SIGVTALRM',
      'SIGXCPU',
      'SIGXFSZ',
      'SIGUSR2',
      'SIGTRAP',
      'SIGSYS',
      'SIGQUIT',
      'SIGIOT' // should detect profiler and enable/disable accordingly.
      // see #21
      // 'SIGPROF'
    );
  }

  if (process.platform === 'linux') {
    module.exports.push('SIGIO', 'SIGPOLL', 'SIGPWR', 'SIGSTKFLT', 'SIGUNUSED');
  }
});

// that are in the direct sync flow of nyc's outputCoverage are
// ignored, since we can never get coverage for them.

var signals$1 = signals;
var EE = events;
/* istanbul ignore if */

if (typeof EE !== 'function') {
  EE = EE.EventEmitter;
}

var emitter;

if (process.__signal_exit_emitter__) {
  emitter = process.__signal_exit_emitter__;
} else {
  emitter = process.__signal_exit_emitter__ = new EE();
  emitter.count = 0;
  emitter.emitted = {};
} // Because this emitter is a global, we have to check to see if a
// previous version of this library failed to enable infinite listeners.
// I know what you're about to say.  But literally everything about
// signal-exit is a compromise with evil.  Get used to it.

if (!emitter.infinite) {
  emitter.setMaxListeners(Infinity);
  emitter.infinite = true;
}

var signalExit = function(cb, opts) {
  assert.equal(
    typeof cb,
    'function',
    'a callback must be provided for exit handler'
  );

  if (loaded === false) {
    load();
  }

  var ev = 'exit';

  if (opts && opts.alwaysLast) {
    ev = 'afterexit';
  }

  var remove = function() {
    emitter.removeListener(ev, cb);

    if (
      emitter.listeners('exit').length === 0 &&
      emitter.listeners('afterexit').length === 0
    ) {
      unload();
    }
  };

  emitter.on(ev, cb);
  return remove;
};

var unload_1 = unload;

function unload() {
  if (!loaded) {
    return;
  }

  loaded = false;
  signals$1.forEach(function(sig) {
    try {
      process.removeListener(sig, sigListeners[sig]);
    } catch (er) {}
  });
  process.emit = originalProcessEmit;
  process.reallyExit = originalProcessReallyExit;
  emitter.count -= 1;
}

function emit(event, code, signal) {
  if (emitter.emitted[event]) {
    return;
  }

  emitter.emitted[event] = true;
  emitter.emit(event, code, signal);
} // { <signal>: <listener fn>, ... }

var sigListeners = {};
signals$1.forEach(function(sig) {
  sigListeners[sig] = function listener() {
    // If there are no other listeners, an exit is coming!
    // Simplest way: remove us and then re-send the signal.
    // We know that this will kill the process, so we can
    // safely emit now.
    var listeners = process.listeners(sig);

    if (listeners.length === emitter.count) {
      unload();
      emit('exit', null, sig);
      /* istanbul ignore next */

      emit('afterexit', null, sig);
      /* istanbul ignore next */

      process.kill(process.pid, sig);
    }
  };
});

var signals_1 = function() {
  return signals$1;
};

var load_1 = load;
var loaded = false;

function load() {
  if (loaded) {
    return;
  }

  loaded = true; // This is the number of onSignalExit's that are in play.
  // It's important so that we can count the correct number of
  // listeners on signals, and don't wait for the other one to
  // handle it instead of us.

  emitter.count += 1;
  signals$1 = signals$1.filter(function(sig) {
    try {
      process.on(sig, sigListeners[sig]);
      return true;
    } catch (er) {
      return false;
    }
  });
  process.emit = processEmit;
  process.reallyExit = processReallyExit;
}

var originalProcessReallyExit = process.reallyExit;

function processReallyExit(code) {
  process.exitCode = code || 0;
  emit('exit', process.exitCode, null);
  /* istanbul ignore next */

  emit('afterexit', process.exitCode, null);
  /* istanbul ignore next */

  originalProcessReallyExit.call(process, process.exitCode);
}

var originalProcessEmit = process.emit;

function processEmit(ev, arg) {
  if (ev === 'exit') {
    if (arg !== undefined) {
      process.exitCode = arg;
    }

    var ret = originalProcessEmit.apply(this, arguments);
    emit('exit', process.exitCode, null);
    /* istanbul ignore next */

    emit('afterexit', process.exitCode, null);
    return ret;
  } else {
    return originalProcessEmit.apply(this, arguments);
  }
}
signalExit.unload = unload_1;
signalExit.signals = signals_1;
signalExit.load = load_1;

var errname_1 = createCommonjsModule(function(module) {
  // In that case, fall back to a deprecated internal.

  let uv;

  if (typeof util.getSystemErrorName === 'function') {
    module.exports = util.getSystemErrorName;
  } else {
    try {
      uv = process.binding('uv');

      if (typeof uv.errname !== 'function') {
        throw new TypeError('uv.errname is not a function');
      }
    } catch (err) {
      console.error(
        "execa/lib/errname: unable to establish process.binding('uv')",
        err
      );
      uv = null;
    }

    module.exports = code => errname(uv, code);
  } // Used for testing the fallback behavior

  module.exports.__test__ = errname;

  function errname(uv, code) {
    if (uv) {
      return uv.errname(code);
    }

    if (!(code < 0)) {
      throw new Error('err >= 0');
    }

    return `Unknown system error ${code}`;
  }
});
var errname_2 = errname_1.__test__;

const alias = ['stdin', 'stdout', 'stderr'];

const hasAlias = opts => alias.some(x => Boolean(opts[x]));

var stdio = opts => {
  if (!opts) {
    return null;
  }

  if (opts.stdio && hasAlias(opts)) {
    throw new Error(
      `It's not possible to provide \`stdio\` in combination with one of ${alias
        .map(x => `\`${x}\``)
        .join(', ')}`
    );
  }

  if (typeof opts.stdio === 'string') {
    return opts.stdio;
  }

  const stdio = opts.stdio || [];

  if (!Array.isArray(stdio)) {
    throw new TypeError(
      `Expected \`stdio\` to be of type \`string\` or \`Array\`, got \`${typeof stdio}\``
    );
  }

  const result = [];
  const len = Math.max(stdio.length, alias.length);

  for (let i = 0; i < len; i++) {
    let value = null;

    if (stdio[i] !== undefined) {
      value = stdio[i];
    } else if (opts[alias[i]] !== undefined) {
      value = opts[alias[i]];
    }

    result[i] = value;
  }

  return result;
};

var execa = createCommonjsModule(function(module) {
  const TEN_MEGABYTES = 1000 * 1000 * 10;

  function handleArgs(cmd, args, opts) {
    let parsed;
    opts = Object.assign(
      {
        extendEnv: true,
        env: {}
      },
      opts
    );

    if (opts.extendEnv) {
      opts.env = Object.assign({}, process.env, opts.env);
    }

    if (opts.__winShell === true) {
      delete opts.__winShell;
      parsed = {
        command: cmd,
        args,
        options: opts,
        file: cmd,
        original: {
          cmd,
          args
        }
      };
    } else {
      parsed = crossSpawn._parse(cmd, args, opts);
    }

    opts = Object.assign(
      {
        maxBuffer: TEN_MEGABYTES,
        buffer: true,
        stripEof: true,
        preferLocal: true,
        localDir: parsed.options.cwd || process.cwd(),
        encoding: 'utf8',
        reject: true,
        cleanup: true
      },
      parsed.options
    );
    opts.stdio = stdio(opts);

    if (opts.preferLocal) {
      opts.env = npmRunPath.env(
        Object.assign({}, opts, {
          cwd: opts.localDir
        })
      );
    }

    if (opts.detached) {
      // #115
      opts.cleanup = false;
    }

    if (
      process.platform === 'win32' &&
      path.basename(parsed.command) === 'cmd.exe'
    ) {
      // #116
      parsed.args.unshift('/q');
    }

    return {
      cmd: parsed.command,
      args: parsed.args,
      opts,
      parsed
    };
  }

  function handleInput(spawned, input) {
    if (input === null || input === undefined) {
      return;
    }

    if (isStream_1(input)) {
      input.pipe(spawned.stdin);
    } else {
      spawned.stdin.end(input);
    }
  }

  function handleOutput(opts, val) {
    if (val && opts.stripEof) {
      val = stripEof(val);
    }

    return val;
  }

  function handleShell(fn, cmd, opts) {
    let file = '/bin/sh';
    let args = ['-c', cmd];
    opts = Object.assign({}, opts);

    if (process.platform === 'win32') {
      opts.__winShell = true;
      file = process.env.comspec || 'cmd.exe';
      args = ['/s', '/c', `"${cmd}"`];
      opts.windowsVerbatimArguments = true;
    }

    if (opts.shell) {
      file = opts.shell;
      delete opts.shell;
    }

    return fn(file, args, opts);
  }

  function getStream(process, stream$$1, { encoding, buffer, maxBuffer }) {
    if (!process[stream$$1]) {
      return null;
    }

    let ret;

    if (!buffer) {
      // TODO: Use `ret = util.promisify(stream.finished)(process[stream]);` when targeting Node.js 10
      ret = new Promise((resolve, reject) => {
        process[stream$$1].once('end', resolve).once('error', reject);
      });
    } else if (encoding) {
      ret = getStream_1(process[stream$$1], {
        encoding,
        maxBuffer
      });
    } else {
      ret = getStream_1.buffer(process[stream$$1], {
        maxBuffer
      });
    }

    return ret.catch(err => {
      err.stream = stream$$1;
      err.message = `${stream$$1} ${err.message}`;
      throw err;
    });
  }

  function makeError(result, options) {
    const { stdout, stderr } = result;
    let err = result.error;
    const { code, signal } = result;
    const { parsed, joinedCmd } = options;
    const timedOut = options.timedOut || false;

    if (!err) {
      let output = '';

      if (Array.isArray(parsed.opts.stdio)) {
        if (parsed.opts.stdio[2] !== 'inherit') {
          output += output.length > 0 ? stderr : `\n${stderr}`;
        }

        if (parsed.opts.stdio[1] !== 'inherit') {
          output += `\n${stdout}`;
        }
      } else if (parsed.opts.stdio !== 'inherit') {
        output = `\n${stderr}${stdout}`;
      }

      err = new Error(`Command failed: ${joinedCmd}${output}`);
      err.code = code < 0 ? errname_1(code) : code;
    }

    err.stdout = stdout;
    err.stderr = stderr;
    err.failed = true;
    err.signal = signal || null;
    err.cmd = joinedCmd;
    err.timedOut = timedOut;
    return err;
  }

  function joinCmd(cmd, args) {
    let joinedCmd = cmd;

    if (Array.isArray(args) && args.length > 0) {
      joinedCmd += ' ' + args.join(' ');
    }

    return joinedCmd;
  }

  module.exports = (cmd, args, opts) => {
    const parsed = handleArgs(cmd, args, opts);
    const { encoding, buffer, maxBuffer } = parsed.opts;
    const joinedCmd = joinCmd(cmd, args);
    let spawned;

    try {
      spawned = child_process.spawn(parsed.cmd, parsed.args, parsed.opts);
    } catch (err) {
      return Promise.reject(err);
    }

    let removeExitHandler;

    if (parsed.opts.cleanup) {
      removeExitHandler = signalExit(() => {
        spawned.kill();
      });
    }

    let timeoutId = null;
    let timedOut = false;

    const cleanup = () => {
      if (timeoutId) {
        clearTimeout(timeoutId);
        timeoutId = null;
      }

      if (removeExitHandler) {
        removeExitHandler();
      }
    };

    if (parsed.opts.timeout > 0) {
      timeoutId = setTimeout(() => {
        timeoutId = null;
        timedOut = true;
        spawned.kill(parsed.opts.killSignal);
      }, parsed.opts.timeout);
    }

    const processDone = new Promise(resolve => {
      spawned.on('exit', (code, signal) => {
        cleanup();
        resolve({
          code,
          signal
        });
      });
      spawned.on('error', err => {
        cleanup();
        resolve({
          error: err
        });
      });

      if (spawned.stdin) {
        spawned.stdin.on('error', err => {
          cleanup();
          resolve({
            error: err
          });
        });
      }
    });

    function destroy() {
      if (spawned.stdout) {
        spawned.stdout.destroy();
      }

      if (spawned.stderr) {
        spawned.stderr.destroy();
      }
    }

    const handlePromise = () =>
      pFinally(
        Promise.all([
          processDone,
          getStream(spawned, 'stdout', {
            encoding,
            buffer,
            maxBuffer
          }),
          getStream(spawned, 'stderr', {
            encoding,
            buffer,
            maxBuffer
          })
        ]).then(arr => {
          const result = arr[0];
          result.stdout = arr[1];
          result.stderr = arr[2];

          if (result.error || result.code !== 0 || result.signal !== null) {
            const err = makeError(result, {
              joinedCmd,
              parsed,
              timedOut
            }); // TODO: missing some timeout logic for killed
            // https://github.com/nodejs/node/blob/master/lib/child_process.js#L203
            // err.killed = spawned.killed || killed;

            err.killed = err.killed || spawned.killed;

            if (!parsed.opts.reject) {
              return err;
            }

            throw err;
          }

          return {
            stdout: handleOutput(parsed.opts, result.stdout),
            stderr: handleOutput(parsed.opts, result.stderr),
            code: 0,
            failed: false,
            killed: false,
            signal: null,
            cmd: joinedCmd,
            timedOut: false
          };
        }),
        destroy
      );

    crossSpawn._enoent.hookChildProcess(spawned, parsed.parsed);

    handleInput(spawned, parsed.opts.input);

    spawned.then = (onfulfilled, onrejected) =>
      handlePromise().then(onfulfilled, onrejected);

    spawned.catch = onrejected => handlePromise().catch(onrejected);

    return spawned;
  }; // TODO: set `stderr: 'ignore'` when that option is implemented

  module.exports.stdout = (...args) =>
    module.exports(...args).then(x => x.stdout); // TODO: set `stdout: 'ignore'` when that option is implemented

  module.exports.stderr = (...args) =>
    module.exports(...args).then(x => x.stderr);

  module.exports.shell = (cmd, opts) => handleShell(module.exports, cmd, opts);

  module.exports.sync = (cmd, args, opts) => {
    const parsed = handleArgs(cmd, args, opts);
    const joinedCmd = joinCmd(cmd, args);

    if (isStream_1(parsed.opts.input)) {
      throw new TypeError('The `input` option cannot be a stream in sync mode');
    }

    const result = child_process.spawnSync(
      parsed.cmd,
      parsed.args,
      parsed.opts
    );
    result.code = result.status;

    if (result.error || result.status !== 0 || result.signal !== null) {
      const err = makeError(result, {
        joinedCmd,
        parsed
      });

      if (!parsed.opts.reject) {
        return err;
      }

      throw err;
    }

    return {
      stdout: handleOutput(parsed.opts, result.stdout),
      stderr: handleOutput(parsed.opts, result.stderr),
      code: 0,
      failed: false,
      signal: null,
      cmd: joinedCmd,
      timedOut: false
    };
  };

  module.exports.shellSync = (cmd, opts) =>
    handleShell(module.exports.sync, cmd, opts);
});
var execa_1 = execa.stdout;
var execa_2 = execa.stderr;
var execa_3 = execa.shell;
var execa_4 = execa.sync;
var execa_5 = execa.shellSync;

/** Check if string is Hash string
 * @param str string that need to be checked
 * @return true if hash string
 */
const isHash = str => /^([a-f0-9]{5,50})|(HEAD([~^]\d*)*)$/gm.test(str);
/** Specific exception for this program
 * Why? Because you need beutiful error message 😆
 * @param {Panic} panic panic object
 * @throws beutiful error message
 */
const becomePanic = panic => {
  throw new Error(
    `${panic.operation.name}: ${panic.operation.data}` +
      `\nHas multiple Id: ${panic.data}` +
      '\n\nFor more info try:' +
      `\n${panic.suggestion}`
  );
};
/** Helper for parsing various string */
const parse$1 = {
  git: {
    /** This will parse output string from `git ls-tree`
     * @see https://git-scm.com/docs/git-ls-tree#_output_format
     * @param output <mode> SP <type> SP <object> TAB <file>
     * @example const {mode,type,sha,name} = parse.git.lsTree('040000 tree c7584b4\t.circleci')
     */
    lsTree(output) {
      // <mode> SP <type> SP <object> TAB <file>
      const [mode, type, ObjectAndFolder] = output.split(' ');
      const [sha, name] = ObjectAndFolder.split('\t');
      return { mode, type, sha, name };
    }
  }
};

//#region based on `git log` flags
const GitLog = {
  abbreviated: true,
  maxCount: 1
};
//#endregion
//#region based on `git ls-tree` flags
const GitListBlob = {
  abbreviated: true,
  recurse: true
};
const GitListTree = {
  abbreviated: true,
  recurse: true
};
const GitListCommit = {
  abbreviated: true
};
//#endregion

/** Get commit hash from commit-message
 * Be careful it will return either individual or array of string
 * @param message commit message
 * @param options objects based on `git log` flags
 * @return Git Object hash of the commit
 */
function fromCommit(message, options = GitLog) {
  const { abbreviated, maxCount } = options;
  const format = abbreviated ? '%h' : '%H';
  const commitIds = execa_4(
    'git',
    [
      'log',
      `--grep='${message}'`,
      `--max-count ${maxCount}`,
      `--pretty='format:${format}'`
    ],
    { shell: true }
  ).stdout.split('\n');
  if (!maxCount)
    throw new Error(
      '`--max-count 0` will not output anything. Are you sure about this?'
    );
  else if (maxCount === 1) return commitIds[0];
  else return commitIds;
}

/** Get blob hash from a file at specific commit
 * @param file filename or filepath. Filepath must be relative path
 * @param commit commit-id (SHA-1) or commit-message
 * @param options objects based on `git ls-tree` flags
 * @return Git Object hash (blob) of the file
 */
function fromFile(file, commit, options = GitListBlob) {
  const getObjSHA = commitId => {
    const { abbreviated, recurse } = options;
    const trees = execa_4(
      'git',
      ['ls-tree', commitId, abbreviated ? '--abbrev' : '', recurse ? '-r' : ''],
      { shell: true }
    )
      .stdout.split('\n')
      .map(output => parse$1.git.lsTree(output));
    return trees.filter(t => t.type !== 'commit' && t.name === file)[0].sha;
  };
  if (isHash(commit)) return getObjSHA(commit);
  else return getObjSHA(fromCommit(commit));
}

/** Get tree hash from a folder at specific commit
 * @param folder folder name or path. Path must be relative path
 * @param commit commit-id (SHA-1) or commit-message
 * @param options objects based on `git ls-tree` flags
 * @return Git Object hash (tree) of the folder
 */
function fromFolder(folder, commit, options = GitListTree) {
  const getObjSHA = commitId => {
    const { abbreviated, recurse } = options;
    const trees = execa_4(
      'git',
      [
        'ls-tree',
        commitId,
        abbreviated ? '--abbrev' : '',
        recurse ? '-r' : '',
        '-d'
      ],
      { shell: true }
    )
      .stdout.split('\n')
      .map(output => parse$1.git.lsTree(output));
    return trees.filter(t => t.type !== 'commit' && t.name === folder)[0].sha;
  };
  if (isHash(commit)) return getObjSHA(commit);
  else return getObjSHA(fromCommit(commit));
}

/** Get commit hash from a submodule at specific commit
 * @param submoduleFolder folder name or path of the git submodule. Path must be relative path
 * @param commit commit-id (SHA-1) or commit-message
 * @param options objects based on `git ls-tree` flags
 * @return Git Object hash (commit) of the submodule
 */
function fromSubmodule(submoduleFolder, commit, options = GitListCommit) {
  const getObjSHA = commitId => {
    const { abbreviated } = options;
    const trees = execa_4(
      'git',
      ['ls-tree', commitId, abbreviated ? '--abbrev' : '', '-r', '-d'],
      { shell: true }
    )
      .stdout.split('\n')
      .map(output => parse$1.git.lsTree(output));
    return trees.filter(
      t => t.type === 'commit' && t.name === submoduleFolder
    )[0].sha;
  };
  if (isHash(commit)) return getObjSHA(commit);
  else return getObjSHA(fromCommit(commit));
}

exports.execa_4 = execa_4;
exports.isHash = isHash;
exports.fromCommit = fromCommit;
exports.fromFile = fromFile;
exports.fromFolder = fromFolder;
exports.fromSubmodule = fromSubmodule;
exports.becomePanic = becomePanic;
