/** Class Handler to make 🆒 semantic API style 😎 */
export default class {
    private opts;
    private ops?;
    private notes?;
    constructor(opts: GitNotes.Options, ops?: "add" | "append" | "overwriteWith" | undefined, notes?: string | undefined);
    /** Insert/Read notes at specific git hash-object
     * @param sha hash-object of commit, blob, or tree
     * @returns chainable function **if possible**
     */
    at(sha: string): void | GitNotes.Operation;
    /** Insert/Read notes at specific file
     * @param message must be string of the message
     * @returns chainable function **if possible**
     */
    atCommit: (message: string) => void | GitNotes.Operation;
    /** Insert/Read notes at specific file
     * @param file can be filename or relative filepath
     * @param commit can be commit-id (SHA-1) or commit-message
     * @returns chainable function **if possible**
     */
    atFile(file: string, commit: string): void | GitNotes.Operation;
    /** Insert/Read notes at specific folder
     * @param tree can be foldername or relativepath
     * @param commit can be commit-id (SHA-1) or commit-message
     * @returns chainable function **if possible**
     */
    atFolder(tree: string, commit: string): void | GitNotes.Operation;
    /** Insert/Read notes at specific submodule
     * @todo Need test 🙂
     *
     * @param submodule must be relativepath
     * @param commit can be commit-id (SHA-1) or commit-message
     * @returns chainable function **if possible**
     */
    atSubmodule(submodule: string, commit: string): void | GitNotes.Operation;
    private getCommitSHA;
}
